#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals

"""배경 색"""
red = u"\033[41m";           #: 빨강색
green = u"\033[42m";			 #: 초록색
yellow = u"\033[43m";        #: 노랑색
blue = u"\033[44m";          #: 파랑색
magenta = u"\033[45m";       #: 자홍색
cyan = u"\033[46m";          #: 시안색
darkgray = u"\033[100m";     #: 밝은 회색
lightgreen = u"\033[102m";   #: 밝은 초록색
